"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = __importDefault(require("@babel/types"));
const css_to_react_native_1 = require("css-to-react-native");
const style_table_1 = __importDefault(require("./style-table"));
const supportUnit = ['rpx', 'pt', 'px', 'vh', 'vw'];
/**
 * 插入import
 * 判断 import 是否有同名
 * 如果没有则插入一条新的
 * 如果有，
 * 则判断是否有导出同名的 ident,
 * 如果有则跳过，
 * 如果没有则替换
 *
 * @param {*} bpath ast path
 * @param {*} source 依赖名
 * @param {*} ident 导出字段名
 */
function injectImportExpression(bpath, source, ident) {
    const program = bpath.findParent((parentNode) => types_1.default.isProgram(parentNode));
    const find = (callback) => program.get('body').find(callback);
    const firstImportNodePath = find((nodePath) => types_1.default.isImportDeclaration(nodePath.node));
    const matchedLibraryPath = find((nodePath) => types_1.default.isImportDeclaration(nodePath.node) && nodePath.node.source.value === source);
    const generatorImportStrcut = (idents) => types_1.default.importDeclaration(idents.map((curId) => types_1.default.importSpecifier(types_1.default.identifier(curId), types_1.default.identifier(curId))), types_1.default.stringLiteral(source));
    if (!firstImportNodePath)
        return;
    if (!matchedLibraryPath) {
        firstImportNodePath.replaceWithMultiple([
            firstImportNodePath.node,
            generatorImportStrcut([ident]),
        ]);
        return;
    }
    const importNode = matchedLibraryPath.node;
    if (!importNode.specifiers) {
        console.log('依赖没有 specifier, 请检查', source);
        return;
    }
    const existIdentifier = importNode.specifiers.find((v) => v.local.name === ident);
    // 已经导入了，则跳过
    if (existIdentifier)
        return;
    importNode.specifiers.push(types_1.default.importSpecifier(types_1.default.identifier(ident), types_1.default.identifier(ident)));
}
exports.default = () => {
    return {
        name: 'babel-plugin-react-native-style',
        visitor: {
            ObjectProperty(path) {
                var _a;
                if (!style_table_1.default.includes((_a = path.node.key) === null || _a === void 0 ? void 0 : _a.name))
                    return;
                if (!types_1.default.isStringLiteral(path.node.value) || !types_1.default.isNumberLiteral)
                    return;
                const styleSheet = css_to_react_native_1.getStylesForProperty(path.node.key.name, path.node.value.value);
                const importLibrarys = new Set();
                const nodes = Object.keys(styleSheet).map((key) => {
                    const value = styleSheet[key];
                    const isNumber = Number(value) === Number(value);
                    let propertyValue = types_1.default.stringLiteral(value);
                    if (isNumber) {
                        propertyValue = types_1.default.numericLiteral(Number(value));
                    }
                    else if (supportUnit.some((unit) => value.endsWith(unit))) {
                        for (let index = 0; index < supportUnit.length; index++) {
                            const unit = supportUnit[index];
                            const variName = '__$' + unit;
                            importLibrarys.add(variName);
                            propertyValue = types_1.default.callExpression(types_1.default.identifier(variName), [
                                types_1.default.numericLiteral(Number(value)),
                            ]);
                        }
                    }
                    return types_1.default.objectProperty(types_1.default.identifier(key), propertyValue);
                });
                importLibrarys.forEach((vari) => {
                    injectImportExpression(path, 'react-native-style-runtime', vari);
                });
                path.replaceWithMultiple(nodes);
            },
        },
    };
};
//# sourceMappingURL=index.js.map